package kubernetes

import (
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	"k8s.io/api/core/v1"
)

func flattenReplicationControllerSpec(in v1.ReplicationControllerSpec, d *schema.ResourceData) ([]interface{}, error) {
	att := make(map[string]interface{})
	att["min_ready_seconds"] = in.MinReadySeconds

	if in.Replicas != nil {
		att["replicas"] = *in.Replicas
	}

	if in.Selector != nil {
		att["selector"] = in.Selector
	}

	if in.Template != nil {
		podSpec, err := flattenPodSpec(in.Template.Spec)
		if err != nil {
			return nil, err
		}
		template := make(map[string]interface{})
		template["spec"] = podSpec
		template["metadata"] = flattenMetadata(in.Template.ObjectMeta, d)
		att["template"] = []interface{}{template}
	}

	return []interface{}{att}, nil
}

func expandReplicationControllerSpec(rc []interface{}) (*v1.ReplicationControllerSpec, error) {
	obj := &v1.ReplicationControllerSpec{}
	if len(rc) == 0 || rc[0] == nil {
		return obj, nil
	}
	in := rc[0].(map[string]interface{})
	obj.MinReadySeconds = int32(in["min_ready_seconds"].(int))
	obj.Replicas = ptrToInt32(int32(in["replicas"].(int)))
	obj.Selector = expandStringMap(in["selector"].(map[string]interface{}))

	template, err := expandReplicationControllerTemplate(in["template"].([]interface{}))
	if err != nil {
		return obj, err
	}

	obj.Template = template

	return obj, nil
}

func expandReplicationControllerTemplate(rct []interface{}) (*v1.PodTemplateSpec, error) {
	obj := &v1.PodTemplateSpec{}
	in := rct[0].(map[string]interface{})
	metadata := in["metadata"].([]interface{})
	obj.ObjectMeta = expandMetadata(metadata)

	podSpec, err := expandPodSpec(in["spec"].([]interface{}))
	if err != nil {
		return obj, err
	}
	obj.Spec = *podSpec

	return obj, nil
}
